function [p,A]=pLUinplace(A)
% permuted LU factorization
%   see Golub G.H & Van Loan C.F., Matrix Computations (3rd E.d) Algorithm 3.4.1, page 112 
%     P*A=L*U
% Returns 
%   - A, the modify n-by-n matrix such that L=tril(A,-1)+eye(n), U=triu(A)
%   - p, the permutation indices 1-by-n array such that the n-by-n permutation matrix P is defined by
%        P(k,p(k))=1 for all k in 1:n
%        P(i,j)=0 otherwize
%    in Matlab/Octave to build the P matrix: 
%       P=zeros(n,n); P(sub2ind(size(P),1:n,p))=1;
%       or
%       P=eye(n,n);P=P(p,:);
%       or
%       P=fc_linalg3D.fact.PermMatFromArray(p);
  n=size(A,1);
  assert(size(A,2)==n)
  p=1:n;
  for i=1:n-1
    [~,idx]=max(abs(A(i:n,i)));
    mu=idx(1)+i-1;
    if abs(A(mu,i))> 1e-15
      if i~=mu
        A([i,mu],:)=A([mu,i],:);
        p([i,mu])=p([mu,i]);
      end
      rows=i+1:n;
      A(rows,i)=A(rows,i)/A(i,i);
      A(rows,rows)=A(rows,rows)-A(rows,i)*A(i,rows);
    end
  end
end
